/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_factory.h
 *
 * Defines the cmsg_factory class, which is an abstract factory for the cmsg
 * class. Messages can only be created from P4 headers using the
 * cmsg_factory::new_msg() method.
 *
 * The cmsg_factory class is implemented as a singleton. Use the instance
 * method to retrieve the cmsg_factory instance.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _MSG_FACTORY_H
#define _MSG_FACTORY_H

#include "protocol.h"


/*****************************************************************************
 * FORWARD DECLARATIONS
 *****************************************************************************/
class cmsg;


/*****************************************************************************
 * CMSG_FACTORY CLASS
 *****************************************************************************/
class cmsg_factory {
  public:

  /** instance
   *
   * Returns the only instance of cmsg_factory. This implements the singleton
   * pattern.
   */
  static const cmsg_factory &instance();

  /** new_msg
   *
   * Takes a P4 header and uses it to create an instance of one of the classes
   * derived from cmsg. This object is *dynamically allocated*, and a pointer
   * is returned.
   *
   * Also works as a copy factory.
   */
  cmsg *new_msg(const p4_header &hdr) const;
  cmsg *new_msg(const cmsg &msg) const;


  protected:
  /** ctor
   */
  cmsg_factory() {}

  /** dtor
   */
  ~cmsg_factory() {}
};

inline const cmsg_factory &cmsg_factory::instance()
{
  static cmsg_factory fact;

  return fact;
}

#endif  // _MSG_FACTORY_H



