/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file protocol.h
 *
 * This header defines common structures used to implement the P4 protocol.
 * More specifically, it provides constants for identifying different message
 * types, strings for the preamble, and structures for each of the messages.
 * For more information, see the P4 protocol specification.
 * 
 * Last modified 05/19/2002 00:27:19
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _P4_PROTOCOL_H
#define _P4_PROTOCOL_H

#include "sys.h"

#include <cassert>
#include <sys/types.h>  // for uint, ulong, ushort

/*****************************************************************************
 * TYPES
 *****************************************************************************/
typedef unsigned char  uchar;


/*****************************************************************************
 * CONSTANTS
 *****************************************************************************/
#define     kp4_preamble        "P4 CONNECT/"VERSION
#define     kp4_preamble_ack    "P4 OK"

#define     kp4_username_len    20    // includes null terminator!!!
#define     kp4_plugin_name_len 20
#define     kp4_sig_len         20    // length in bytes

// P4 message types
const uchar kp4_msgtype_mask  = 0x0f;
const uchar kp4_hello         = 0x01;
const uchar kp4_ping          = 0x02;
const uchar kp4_ping_ack      = 0x03;
const uchar kp4_connect       = 0x04;
const uchar kp4_discover      = 0x05;
const uchar kp4_lka           = 0x06;
const uchar kp4_lka_ack       = 0x07;
const uchar kp4_data          = 0x08;

// P4 message flags
const uchar kp4_encrypted     = 0x10;
const uchar kp4_broadcast     = 0x20;



/*****************************************************************************
 * HEADER FORMAT STRUCTURES
 *****************************************************************************/

/** p4_header
 *
 * The structure used for the standard P4 message header. Includes fields for
 * flags, hops to live, plugin ID, data length, message ID, username, and
 * RSA digital signature.
 */
struct p4_header {
  uchar  flags;
  uchar  htl;
  ushort plugin_id;
  ulong  data_len;
  double message_id;
  char   username[kp4_username_len];
  char   sig[kp4_sig_len];

  /** ctor (so that flags starts out to be 0)
   */
  p4_header() : flags(0) {}

  /** inline helpers...
   */
  void set_flag(uchar flag);
  void clear_flag(uchar flag);
  bool is_flag_set(uchar flag) const;
  void set_type(uchar type);
  bool is_type(uchar type) const;
  uchar get_type() const;
};


/** p4_hello
 *
 * The structure used for the data of a HELLO message. Includes fields for
 * source IP address and port and number of plugins. One or more p4_plugin_info
 * structs will be sent after this message.
 */
typedef struct {
  ulong  addr;
  ushort port;
  ushort num_plugins;
} p4_hello;


/** p4_discover
 *
 * The structure used for the data of a DISCOVER message. Includes fields for
 * source IP address and port.
 */
typedef struct {
  char username[kp4_username_len];
} p4_discover;


/** p4_lka_ack
 *
 * The structure used for the data of a LKA_ACK message. Includes a field for
 * the number of entries. This struct isn't absolutely necessary, but it's given
 * so that the type for the num_entries field is known.
 */
typedef struct {
  ulong num_entries;
} p4_lka_ack;



/*****************************************************************************
 * OTHER STRUCTURES
 *****************************************************************************/

/** lka_entry
 *
 * The format of an entry in the LKA table. Includes username, IP address, port,
 * and number of plugins the user is running.
 */
typedef struct {
  char   username[kp4_username_len];
  ulong  addr;
  ushort port;
  ushort num_plugins;
} lka_entry;


/** plugin_info
 *
 * Information about a plugin - its name and id.
 */
typedef struct {
  ushort id;
} plugin_info;



/*****************************************************************************
 * INLINE METHODS
 *****************************************************************************/
inline void p4_header::set_flag(uchar flag)
{
  flags |= flag;
}

inline void p4_header::clear_flag(uchar flag)
{
  flags &= (~flag);
}

inline bool p4_header::is_flag_set(uchar flag) const
{
  return ((flags & flag) != 0);
}

inline void p4_header::set_type(uchar type)
{
  assert((type & kp4_msgtype_mask) == type);

  flags &= (~kp4_msgtype_mask);  // clear type bits
  flags |= (type);               // set type bits
}

inline bool p4_header::is_type(uchar type) const
{
  return ((flags & kp4_msgtype_mask) == type);
}

inline uchar p4_header::get_type() const
{
  return flags & kp4_msgtype_mask;
}

#endif  // _P4_PROTOCOL_H
